#ifndef DocumentTypeImpl_HEADER_GUARD_
#define DocumentTypeImpl_HEADER_GUARD_

/*
 * The Apache Software License, Version 1.1
 * 
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 * 
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache\@apache.org.
 * 
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation, and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.ibm.com .  For more information
 * on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/**
 * $Log: DocumentTypeImpl.hpp,v $
 * Revision 1.1.1.1  1999/11/09 01:08:44  twl
 * Initial checkin
 *
 * Revision 1.3  1999/11/08 20:44:25  rahul
 * Swat for adding in Product name and CVS comment log variable.
 *
 */

//
//  This file is part of the internal implementation of the C++ XML DOM.
//  It should NOT be included or used directly by application programs.
//
//  Applications should include the file <dom/DOM.hpp> for the entire
//  DOM API, or DOM_*.hpp for individual DOM classes, where the class
//  name is substituded for the *.
//



#include <util/XML4CDefs.hpp>
#include "NodeImpl.hpp"

class NamedNodeMapImpl;

class DocumentTypeImpl: public NodeImpl {
private:
    NamedNodeMapImpl *entities;
    NamedNodeMapImpl *notations;
    NamedNodeMapImpl *elements;
    
public:
    DocumentTypeImpl(DocumentImpl *, const DOMString &);
    DocumentTypeImpl(const DOMString &qualifiedName,	//DOM Level 2
	const DOMString &publicID, const DOMString &systemID,
	const DOMString &internalSubset);
    DocumentTypeImpl(const DocumentTypeImpl &other, bool deep=false);
    virtual ~DocumentTypeImpl();
    virtual bool isDocumentTypeImpl();
    
    virtual NodeImpl *         cloneNode(bool deep);
    virtual NamedNodeMapImpl * getElements();
    virtual NamedNodeMapImpl * getEntities();
    virtual DOMString          getName();
    virtual NamedNodeMapImpl * getNotations();
    virtual void               setNodeValue(const DOMString &arg); 
    virtual void               setReadOnly(bool readOnly, bool deep);

    //Introduced in DOM Level 2
    DOMString publicID;
    DOMString systemID;
    DOMString internalSubset;
    virtual DOMString     getPublicID();
    virtual DOMString     getSystemID();
    virtual DOMString     getInternalSubset();
    virtual void setOwnerDocument(DocumentImpl *docImpl); //internal use only
};

#endif
